
//#define N64_PIN 8 // = B0? - not that it matters, I'm using PB0 regardless
#define N64_HIGH DDRB &= ~0x01
#define N64_LOW DDRB |= 0x01
#define N64_QUERY (PINB & 0x01)


// output n64 command through B1
#define N642_HIGH DDRB &= ~(1<<PB1)
#define N642_LOW DDRB |= (1<<PB1)


#define PSdata      PD3
#define PScommand   PD5
#define PSclock     PD2
#define PSattention PD6

#define sbi(x,y)  x|=(1<<y)
#define cbi(x,y)  x&=~(1<<y)


#define LED_ON PORTD |=(1<<PD0)
#define LED_OFF PORTD &= ~(1 << PD0);

#define abs(x) ((x)<0 ? -(x) : (x))

#ifndef F_CPU
#define F_CPU 16000000 // 16MHz
#endif


/* Includes: */
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/power.h>
#include <avr/interrupt.h>
#include <stdbool.h>
#include <string.h>
#include <util/delay.h>
#include "psx.c"

#include "crc_table.h"
/*
char offset[71]={
	0,0,0,0,0,
	1,1,1,1,1,1,
	2,2,2,2,2,
	3,3,3,3,3,3,
	4,4,4,4,4,
	5,5,5,5,5,5,
	6,6,6,6,6,
	7,7,7,7,7,7,
	8,8,8,8,8,
	9,9,9,9,9,9,
	10,10,10,10,10,
	11,11,11,11,11,11,
	12,12,12,12,12
};
*/
char offset[73]={
0,0,0,
1,1,1,1,1,
2,2,2,2,2,
3,3,3,3,3,
4,4,4,4,4,
5,5,5,5,5,
6,6,6,6,6,
7,7,7,7,7,
8,8,8,8,8,
9,9,9,9,9,
10,10,10,10,10,
11,11,11,11,11,
12,12,12,12,12,
13,13,13,13,13,
14,14,14,14,14
};




bool okToRead=1;
bool frame=0;

char but[2];     //state of button
char butonos[2]; //button pressed oneshot
char butl[2];    //button last pass
char turbo; //state of relay or led
char halfStick; //state of relay or led

//Common structure for last read data
static uint8_t _data[21];

static char n64_raw_dump[281]; // maximum recv is 1+2+32 bytes + 1 bit
// n64_raw_dump does /not/ include the command byte. That gets pushed into
// n64_command:
static unsigned char n64_command;

static unsigned char n64_buffer[33];
char N64_raw_dump[33]; // 1 received bit per byte

/* Function Prototypes: */
void SetupHardware(void);
static void n64_send(unsigned char *buffer, char length, bool wide_stop);
static void get_n64_command(void);
int main(void);
void loop(void);
static void psx_to_64(void);
void N64_get(void);
void noInterrupts(void);
void interrupts(void);


void noInterrupts(void){
	cli();                // disable global interrupts
}
void interrupts(void){
	sei();                // enable interrupts
}

void AnalogMode(void){
  //Enter Config Mode
  uint8_t enter_config_command[] = {0x01, 0x43, 0x00, 0x01, 0x00};
  _psx_send_command(enter_config_command, 5);

  // Lock to Analog Mode on Stick
  uint8_t lock_analog_mode_command[] = {0x01, 0x44, 0x00, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00};
  _psx_send_command(lock_analog_mode_command, 9);


  // Lock Rumble
  uint8_t lock_rumble_mode_command[] = {0x01, 0x4D, 0x00, 0x00, 0x01, 0xFF, 0xFF, 0xFF, 0xFF};
  _psx_send_command(lock_rumble_mode_command, 9);


  //Exit config mode
  uint8_t exit_config_command[] = {0x01, 0x43, 0x00, 0x00, 0x5A, 0x5A, 0x5A, 0x5A, 0x5A};
  _psx_send_command(exit_config_command, 9);
	
}

/** Main program entry point. This routine contains the overall program flow, including initial
 *  setup of all components and the main program loop.
 */
int main(void)
{
    noInterrupts();
	SetupHardware();

    for (;;)
	{

/*

*/

//	 if(psx_button(PSB_TRIANGLE)){stickMode=0;}
//	 if(psx_button(PSB_CIRCLE)){stickMode=1;}
		frame = 1-frame;

	    loop(); // do the n64 stuff
		if(okToRead == 1){
			
			psx_read_gamepad(); // wired pad
			
			but[0] = psx_button(PSB_SELECT);
			but[1] = psx_button(PSB_TRIANGLE);
			for(char t=0; t<2; t++){
				butonos[t]=but[t] && !butl[t];      //button pressed one shot (edge)
				butl[t]=but[t];                  //remember last pass
			}
			turbo ^= butonos[0];          //toggle relay var on button oneshot
			halfStick ^= butonos[1];          //toggle relay var on button oneshot
			
			if(_data[1]>>4 !=0x7){
				psx_init(&PORTD, 3,	&PORTD, 2,	&PORTD, 5,	&PORTD, 6);
			}
		}

	}
}

/** Configures the board hardware and chip peripherals for the demo's functionality. */
void SetupHardware()
{
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();
	clock_prescale_set(clock_div_1);

	psx_init(&PORTD, 3,	&PORTD, 2,	&PORTD, 5,	&PORTD, 6);

  // Communication with the N64 on this pin
  PORTB &=~ (1 << PB0); // Pin n goes low
  DDRB |= (1 << PB0);        // PB0 Input
  N64_HIGH;
  N642_LOW;
  
  // LED
  DDRD |=  (1 << PD0);         //Set PortD pin as an output
  LED_OFF;
}




static void psx_to_64()
{
   // clear it out
    memset(n64_buffer, 0, sizeof(n64_buffer));

   //                                              PSX -> 64
   n64_buffer[0] |= psx_button(PSB_CROSS)  << 7;  // X -> A
   n64_buffer[0] |= psx_button(PSB_SQUARE) << 6;  //[] -> B
   n64_buffer[0] |= (psx_button(PSB_L2) | psx_button(PSB_R2))     << 5;  //L2/R2 -> Z
   n64_buffer[0] |= (psx_button(PSB_START)  << 4);  // S -> S
   n64_buffer[0] |= psx_button(PSB_PAD_UP)   << 3; // D pad up
   n64_buffer[0] |= psx_button(PSB_PAD_DOWN) << 2; // D pad down
   n64_buffer[0] |= psx_button(PSB_PAD_LEFT) << 1; // D pad left
   n64_buffer[0] |= psx_button(PSB_PAD_RIGHT)    ; // D pad right

	if(frame==0 && turbo == 1){
		n64_buffer[0] |= (1 << 4);  // S -> S
	}

   // Second byte to N64 should contain:
   // 0, 0, L, R, Cup, Cdown, Cleft, Cright

   n64_buffer[1] |= psx_button(PSB_L1)     << 5;  //L2 -> Z
   n64_buffer[1] |= psx_button(PSB_R1)  << 4;  // S -> S

   // C buttons are tricky, translate the C stick values to determine which C
   // buttons are "pressed"
   // Analog sticks are a value 0-255 with the center at 128 the maximum and
   // minimum values seemed to vary a bit, but we only need to choose a
   // threshold here
   if (psx_stick(PSS_RY) > 0xB0) {
	   // C-down
	   n64_buffer[1] |= 0x04;
   }
   if (psx_stick(PSS_RY) < 0x50) {
	   // C-up
	   n64_buffer[1] |= 0x08;
   }
   if (psx_stick(PSS_RX) < 0x50) {
	   // C-left
	   n64_buffer[1] |= 0x02;
   }
   if (psx_stick(PSS_RX) > 0xB0) {
	   // C-right
	   n64_buffer[1] |= 0x01;
   }

//    n64_buffer[2] = (psx_stick(PSS_LX)/1.5)-85;
//    n64_buffer[3] = (-psx_stick(PSS_LY)/1.5)+85;

//    int myX = (psx_stick(PSS_LX)/1.2)-106;
//    int myY = (-psx_stick(PSS_LY)/1.2)+106;
    int myX = (psx_stick(PSS_LX)/1.1)-115;
    int myY = (-psx_stick(PSS_LY)/1.1)+115;

	if(halfStick){
		LED_ON;
		myX = myX >> 1;
		myY = myY >> 1;
		}else{
		LED_OFF;
	}


	if(myX>=0){
		myX -= 21;
		if(myX < 0){myX = 0;}
		if(myX > 85){myX = 85;}
	}else{
		myX += 21;
		if(myX > 0){myX = 0;}
		if(myX < -85){myX = -85;}
	}
	
    if(myY>=0){
	    myY -= 21;
	    if(myY < 0){myY = 0;}
		if(myY > 85){myY = 85;}
    }else{
	    myY += 21;
	    if(myY > 0){myY = 0;}
		if(myY < -85){myY = -85;}
    }

	signed char tX = abs(myX);
	signed char tY = abs(myY);
	
	if(tX>71){tX=71;}
	if(tY>71){tY=71;}

	if(myY>( 85-offset[tX])){myY= 85-offset[tX];}
	if(myY<(-85+offset[tX])){myY=-85+offset[tX];}
	if(myX>( 85-offset[tY])){myX= 85-offset[tY];}
	if(myX<(-85+offset[tY])){myX=-85+offset[tY];}

	n64_buffer[2] = myX;
    n64_buffer[3] = myY;

}

static void n64_send(unsigned char *buffer, char length, bool wide_stop)
{
	
	N642_HIGH;

	asm volatile (";Starting N64 Send Routine");
	// Send these bytes
	char bits;
	
	// This routine is very carefully timed by examining the assembly output.
	// Do not change any statements, it could throw the timings off
	//
	// We get 16 cycles per microsecond, which should be plenty, but we need to
	// be conservative. Most assembly ops take 1 cycle, but a few take 2
	//
	// I use manually constructed for-loops out of gotos so I have more control
	// over the outputted assembly. I can insert nops where it was impossible
	// with a for loop
	
	asm volatile (";Starting outer for loop");
	outer_loop:
	{
		asm volatile (";Starting inner for loop");
		bits=8;
		inner_loop:
		{
			// Starting a bit, set the line low
			asm volatile (";Setting line to low");
			N64_LOW; // 1 op, 2 cycles

			asm volatile (";branching");
			if (*buffer >> 7) {
				asm volatile (";Bit is a 1");
				// 1 bit
				// remain low for 1us, then go high for 3us
				// nop block 1
				asm volatile ("nop\nnop\nnop\nnop\nnop\nnop\nnop\n");
				
				asm volatile (";Setting line to high");
				N64_HIGH;

				// nop block 2
				// we'll wait only 2us to sync up with both conditions
				// at the bottom of the if statement
				asm volatile ("nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				);

				} else {
				asm volatile (";Bit is a 0");
				// 0 bit
				// remain low for 3us, then go high for 1us
				// nop block 3
				asm volatile (
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				"nop\nnop\nnop\nnop\nnop\n"
				);

				asm volatile (";Setting line to high");
				N64_HIGH;

				// wait for 1us
				asm volatile ("; end of conditional branch, need to wait 1us more before next bit");
				
			}
			// end of the if, the line is high and needs to remain
			// high for exactly 16 more cycles, regardless of the previous
			// branch path

			asm volatile (";finishing inner loop body");
			--bits;
			if (bits != 0) {
				// nop block 4
				// this block is why a for loop was impossible
				asm volatile ("nop\nnop\nnop\nnop\nnop\n"
				"nop\n");
				// rotate bits
				asm volatile (";rotating out bits");
				*buffer <<= 1;

				goto inner_loop;
			} // fall out of inner loop
		}
		asm volatile (";continuing outer loop");
		// In this case: the inner loop exits and the outer loop iterates,
		// there are /exactly/ 16 cycles taken up by the necessary operations.
		// So no nops are needed here (that was lucky!)
		--length;
		if (length != 0) {
			++buffer;
			goto outer_loop;
		} // fall out of outer loop
	}

	// send a single stop (1) bit
	// nop block 5
	asm volatile ("nop\nnop\nnop\nnop\n");
	N64_LOW;
	// wait 1 us, 16 cycles, then raise the line
	// take another 3 off for the wide_stop check
	// 16-2-3=11
	// nop block 6
	asm volatile ("nop\nnop\nnop\nnop\nnop\n"
	"nop\nnop\nnop\nnop\nnop\n"
	"nop\n");
	if (wide_stop) {
		asm volatile (";another 1us for extra wide stop bit\n"
		"nop\nnop\nnop\nnop\nnop\n"
		"nop\nnop\nnop\nnop\nnop\n"
		"nop\nnop\nnop\nnop\n");
	}

	N64_HIGH;
	N642_LOW;

}




void loop()
{
    unsigned char data, addr;
	//okToRead = 0;

    // clear out incoming raw data buffer
    // this should be unnecessary
    memset(n64_raw_dump, 0, sizeof(n64_raw_dump));

	//psx_read_gamepad();
	psx_to_64();

    // Wait for incoming 64 command
    // this will block until the N64 sends us a command
    // Wait for incoming 64 command
    // this will block until the N64 sends us a command
    get_n64_command();

    // 0x00 is identify command
    // 0x01 is status
    // 0x02 is read
    // 0x03 is write
    switch (n64_command)
    {
	    case 0x00:
	    case 0xFF:
		okToRead=1;

	    // identify
	    // mutilate the n64_buffer array with our status
	    // we return 0x050001 to indicate we have a rumble pack
	    // or 0x050002 to indicate the expansion slot is empty
	    //
	    // 0xFF I've seen sent from Mario 64 and Shadows of the Empire.
	    // I don't know why it's different, but the controllers seem to
	    // send a set of status bytes afterwards the same as 0x00, and
	    // it won't work without it.
	    n64_buffer[0] = 0x05;
	    n64_buffer[1] = 0x00;
	    n64_buffer[2] = 0x01;

		_delay_us(4);

	    n64_send(n64_buffer, 3, 0);
		
	    //Serial.println("It was 0x00: an identify command");
	    break;
	    case 0x01:
		okToRead=1;
		
	    // blast out the pre-assembled array in n64_buffer
	    n64_send(n64_buffer, 4, 0);

	    //Serial.println("It was 0x01: the query command");
	    break;
	    case 0x02:
		okToRead=0;
	    // A read. If the address is 0x8000, return 32 bytes of 0x80 bytes,
	    // and a CRC byte.  this tells the system our attached controller
	    // pack is a rumble pack

	    // Assume it's a read for 0x8000, which is the only thing it should
	    // be requesting anyways
	    memset(n64_buffer, 0x80, 32);
	    n64_buffer[32] = 0xB8; // CRC

	    n64_send(n64_buffer, 33, 1);
		
	    //Serial.println("It was 0x02: the read command");
	    break;
	    case 0x03:
		okToRead=0;
	    // A write. we at least need to respond with a single CRC byte.  If
	    // the write was to address 0xC000 and the data was 0x01, turn on
	    // rumble! All other write addresses are ignored. (but we still
	    // need to return a CRC)

	    // decode the first data byte (fourth overall byte), bits indexed
	    // at 24 through 31
	    data = 0;
	    data |= (n64_raw_dump[16] != 0) << 7;
	    data |= (n64_raw_dump[17] != 0) << 6;
	    data |= (n64_raw_dump[18] != 0) << 5;
	    data |= (n64_raw_dump[19] != 0) << 4;
	    data |= (n64_raw_dump[20] != 0) << 3;
	    data |= (n64_raw_dump[21] != 0) << 2;
	    data |= (n64_raw_dump[22] != 0) << 1;
	    data |= (n64_raw_dump[23] != 0);

	    // get crc byte, invert it, as per the protocol for
	    // having a memory card attached
	    n64_buffer[0] = crc_repeating_table[data] ^ 0xFF;

	    // send it
	    n64_send(n64_buffer, 1, 1);

	    // end of time critical code
	    // was the address the rumble latch at 0xC000?
	    // decode the first half of the address, bits
	    // 8 through 15
	    addr = 0;
	    addr |= (n64_raw_dump[0] != 0) << 7;
	    addr |= (n64_raw_dump[1] != 0) << 6;
	    addr |= (n64_raw_dump[2] != 0) << 5;
	    addr |= (n64_raw_dump[3] != 0) << 4;
	    addr |= (n64_raw_dump[4] != 0) << 3;
	    addr |= (n64_raw_dump[5] != 0) << 2;
	    addr |= (n64_raw_dump[6] != 0) << 1;
	    addr |= (n64_raw_dump[7] != 0);

	    if (addr == 0xC0) {
		    rumble = (data != 0);
	    }

	    break;

	    default:
		okToRead=1;
	    break;

    }

 //   interrupts();
}

/**
  * Waits for an incoming signal on the N64 pin and reads the command,
  * and if necessary, any trailing bytes.
  * 0x00 is an identify request
  * 0x01 is a status request
  * 0x02 is a controller pack read
  * 0x03 is a controller pack write
  *
  * for 0x02 and 0x03, additional data is passed in after the command byte,
  * which is also read by this function.
  *
  * All data is raw dumped to the n64_raw_dump array, 1 bit per byte, except
  * for the command byte, which is placed all packed into n64_command
  */

static void get_n64_command()
{
    int bitcount;
    char *bitbin = n64_raw_dump;
    int idle_wait;

    n64_command = 0;

    bitcount = 8;

    // wait to make sure the line is idle before
    // we begin listening
    for (idle_wait=16; idle_wait>0; --idle_wait) {
        if (!N64_QUERY) {
            idle_wait = 16;
        }
    }
read_loop:
        // wait for the line to go low
        while (N64_QUERY){}
        // wait approx 2us and poll the line
        asm volatile (
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                );
        if (N64_QUERY)
            n64_command |= 0x01;

        --bitcount;
        if (bitcount == 0)
            goto read_more;

        n64_command <<= 1;

        // wait for line to go high again
        // I don't want this to execute if the loop is exiting, so
        // I couldn't use a traditional for-loop
        while (!N64_QUERY) {}
        goto read_loop;

read_more:
        switch (n64_command)
        {
            case (0x03):
                // write command
                // we expect a 2 byte address and 32 bytes of data
                bitcount = 272 + 1; // 34 bytes * 8 bits per byte
                //Serial.println("command is 0x03, write");
                break;
            case (0x02):
                // read command 0x02
                // we expect a 2 byte address
                bitcount = 16 + 1;
                //Serial.println("command is 0x02, read");
                break;
            case (0x00):
            case (0x01):
            default:
                // get the last (stop) bit
                bitcount = 1;
                break;
        }

        // make sure the line is high. Hopefully we didn't already
        // miss the high-to-low transition
        while (!N64_QUERY) {}
read_loop2:
        // wait for the line to go low
        while (N64_QUERY){}

        // wait approx 2us and poll the line
        asm volatile (
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                      "nop\nnop\nnop\nnop\nnop\n"  
                );
        *bitbin = N64_QUERY;
        ++bitbin;
        --bitcount;
        if (bitcount == 0)
			 return;
		

        // wait for line to go high again
        while (!N64_QUERY) {}
        goto read_loop2;
}

