#define PSclock     PD2
#define PSdata      PD3
#define PSacknolage PD4
#define PScommand   PD5
#define PSattention PD6

//These are our button constants
#define PSB_SELECT      0x0001
#define PSB_L3          0x0002
#define PSB_R3          0x0004
#define PSB_START       0x0008
#define PSB_PAD_UP      0x0010
#define PSB_PAD_RIGHT   0x0020
#define PSB_PAD_DOWN    0x0040
#define PSB_PAD_LEFT    0x0080
#define PSB_L2          0x0100
#define PSB_R2          0x0200
#define PSB_L1          0x0400
#define PSB_R1          0x0800
#define PSB_TRIANGLE    0x1000
#define PSB_CIRCLE      0x2000
#define PSB_CROSS       0x4000
#define PSB_SQUARE      0x8000

//These are stick values
#define PSS_RX 5
#define PSS_RY 6
#define PSS_LX 7
#define PSS_LY 8

//Common structure for last read data
static uint8_t _data[21];

int rumble = 0;


uint16_t psx_buttons() {
	uint16_t buttons = *(uint16_t*)(_data + 3); //Get 2 bytes, comprising data positions 3 and 4.
	return ~buttons;
}

uint8_t psx_button(uint16_t button) {
	uint16_t buttons = psx_buttons();
	return ((buttons & button) > 0);
}

uint8_t psx_stick(unsigned int stick) {
	return _data[stick];
}


unsigned char chk_ana = 0;
int data0=0,data1=0,data2=0,data3=0,data4=0,data5=0,temp=0;

int gameByte(int command)
{
	int i ;
	_delay_us(1);
	int data = 0x00;                             // clear data variable to save setting low bits later.
	for(i=0;i<8;i++)
	{
		if(command & _BV(i))
		{
			PORTD|=(1<<PScommand);//sbi(PORTD, PScommand);
		}// bit bang "command" out on PScommand wire.
		else
		{
			PORTD&=~(1<<PScommand);
		}
		PORTD&=~(1<<PSclock);                             // CLOCK LOW
		_delay_us(1);                                              // wait for output to stabilise
		if((PIND & _BV(PSdata)))
		{
			data|=(1<<i);  // read PSdata pin and store
		}//else cbi(data, i);
		PORTD|=(1<<PSclock);                             // CLOCK HIGH
	}
	PORTD|=(1<<PScommand);

	_delay_us(20);                                                   // wait for ACK to pass.

	return(data);
}

void joystick_config()
{
	// put controller in config mode
	PORTD|=(1<<PScommand);
	PORTD|=(1<<PSclock);
	PORTD&=~(1<<PSattention);

	_data[0] = gameByte(0x01);
	_data[1] = gameByte(0x43);
	_data[2] = gameByte(0x00);
	_data[3] = gameByte(0x01);
	_data[4] = gameByte(0x00);

	PORTD|=(1<<PScommand);
	_delay_ms(1);
	PORTD|=(1<<PSattention);

	_delay_ms(10);

	// put controller in analouge mode
	PORTD|=(1<<PScommand);
	PORTD|=(1<<PSclock);
	PORTD&=~(1<<PSattention);

	_data[0] = gameByte(0x01);
	_data[1] = gameByte(0x44);
	_data[2] = gameByte(0x00);
	_data[3] = gameByte(0x01);
	_data[4] = gameByte(0x03);
	_data[5] = gameByte(0x00);
	_data[6] = gameByte(0x00);
	_data[7] = gameByte(0x00);
	_data[8] = gameByte(0x00);

	PORTD|=(1<<PScommand);
	_delay_ms(1);
	PORTD|=(1<<PSattention);

	_delay_ms(10);

	// exit config mode
	PORTD|=(1<<PScommand);
	PORTD|=(1<<PSclock);
	PORTD&=~(1<<PSattention);

	_data[0] = gameByte(0x01);
	_data[1] = gameByte(0x43);
	_data[2] = gameByte(0x00);
	_data[3] = gameByte(0x00);
	_data[4] = gameByte(0x5A);
	_data[5] = gameByte(0x5A);
	_data[6] = gameByte(0x5A);
	_data[7] = gameByte(0x5A);
	_data[8] = gameByte(0x5A);

	PORTD|=(1<<PScommand);
	_delay_ms(1);
	PORTD|=(1<<PSattention);

	_delay_ms(10);

	// poll controller and check in analouge mode.
	PORTD|=(1<<PScommand);
	PORTD|=(1<<PSclock);
	PORTD&=~(1<<PSattention);

	_data[0] = gameByte(0x01);
	_data[1] = gameByte(0x42);            // the 2nd byte to be returned from the controller should = 0x73 for "red" analouge controller.
	_data[2] = gameByte(0x00);
	_data[3] = gameByte(0x00);
	_data[4] = gameByte(0x00);
	_data[5] = gameByte(0x00);
	_data[6] = gameByte(0x00);
	_data[7] = gameByte(0x00);
	_data[8] = gameByte(0x00);

	PORTD|=(1<<PScommand);
	_delay_ms(1);
	PORTD|=(1<<PSattention);
	_delay_ms(10);

}

void ps2_init()
{
	DDRD|=(1<<PD2);                 // clock. output. (blue)

	DDRD&=~(1<<PD3);                 // data. input. (brown)
	PORTD|=(1<<PD3);               //    enable pullup resistor

	DDRD&=~(1<<PD4);                 // acknolage. input. (green)
	PORTD|=(1<<PD4);               //    enable pullup resistor

	DDRD|=(1<<PD5);                 // command. output. (orange)

	DDRD|=(1<<PD6);                 // attention. output. (yellow)
}

void joystick_poll()
{
	PORTD|=(1<<PScommand);                          // start communication with PSx controller
	PORTD|=(1<<PSclock);
	PORTD&=~(1<<PSattention);

	_data[0] = gameByte(0x01);                                       // bite 0. header.
	_data[1] = gameByte(0x42);                          // bite 1. header. (should possibly put test on this byte to detect unplugging of controller.)
	_data[2] = gameByte(0x00);                                       // bite 2. header.

	_data[3] = gameByte(0x00);                         // bite 3. first data bite.
	_data[4] = gameByte(0x00);                         // bite 4.
	_data[5] = gameByte(0x00);                         // bite 5.
	_data[6] = gameByte(0x00);                         // bite 6.
	_data[7] = gameByte(0x00);                         // bite 7.
	_data[8] = gameByte(0x00);                         // bite 8.
	_data[9] = gameByte(0x00);                         // bite 8.
	_data[10] = gameByte(0x00);                         // bite 9.
	_data[11] = gameByte(0x00);                         // bite 10.
	_data[12] = gameByte(0x00);                         // bite 11.
	_data[13] = gameByte(0x00);                         // bite 12.
	_data[14] = gameByte(0x00);                         // bite 13.
	_data[15] = gameByte(0x00);                         // bite 14.
	_data[16] = gameByte(0x00);                         // bite 15.
	_data[17] = gameByte(0x00);                         // bite 16.
	_data[18] = gameByte(0x00);                         // bite 17.
	_data[19] = gameByte(0x00);                         // bite 18.
	_data[20] = gameByte(0x00);                         // bite 19.

	_delay_us(1);
	PORTD|=(1<<PScommand);                      // close communication with PSx controller
	_delay_us(1);
	PORTD|=(1<<PSattention);                        // all done.
}